@@ -65,15 +65,18 @@ void add_child_fd(int fd, int pid) {
       return;
     }
 }
-int find_server(struct in_addr *sin) {
+int find_server(struct in_addr *sin, unsigned short port) {
   register int i;
   for(i=0;i<MAXSERVERS;i++)
     if(memcmp(sin, &serverstats[i].contact.sin_addr,
-	      sizeof(struct in_addr))==0)
+	      sizeof(struct in_addr))==0 &&
+	port==serverstats[i].contact.sin_port)
       return i;
   return -1;
 }
-int new_session(struct sockaddr_in *sin) {
+static void connect_alarm(int a) {
+}
+int new_session(unsigned int address, unsigned short port) {
   int nsd;
   struct sockaddr_in sain;
   nsd = socket(AF_INET, SOCK_STREAM, 0);
@@ -94,8 +97,11 @@ int new_session(struct sockaddr_in *sin) {
     }
   }
 #endif
-  sain = *sin;
+  sain.sin_addr.s_addr = address;
+  sain.sin_port = port;
   sain.sin_family = AF_INET;
+  signal(SIGALRM, connect_alarm);
+  alarm(3);
   if(connect(nsd, (struct sockaddr *)&sain, sizeof(struct sockaddr_in))<0) {
     if(loglevel & MBLL_NET3) {
       ap_log_error(APLOG_MARK, APLOG_ERR, NULL,
@@ -104,11 +110,18 @@ int new_session(struct sockaddr_in *sin) {
     close(nsd);
     nsd=-1;
   }
+  if(errno == EINTR) {
+    ap_log_error(APLOG_MARK, APLOG_WARNING, NULL,
+                 "connect() timed out");
+    close(nsd);
+    nsd=-1;
+  }
   return nsd;
 }
-static int reserve_session(struct in_addr *request, pid_t pid) {
+static int reserve_session(struct in_addr *request, unsigned short port,
+			   pid_t pid) {
   int i,j,rfd=-1;
-  if((i=find_server(request))==-1) return -1;
+  if((i=find_server(request, port))==-1) return -1;
   for(j=0;j<MAXSESSIONSPERSERVER && rfd<0;j++)
     if(connection_pool[i][j].fd>=0) {
       rfd=connection_pool[i][j].fd;
@@ -118,7 +131,9 @@ static int reserve_session(struct in_addr *request, pid_t pid) {
     }      
   for(j=0;j<MAXSESSIONSPERSERVER && rfd<0;j++)
     if(connection_pool[i][j].fd<0) {
-      connection_pool[i][j].fd = new_session(&serverstats[i].contact);
+      connection_pool[i][j].fd =
+	new_session(serverstats[i].contact.sin_addr.s_addr,
+		    serverstats[i].contact.sin_port);
       connection_pool[i][j].pid = pid;
       rfd=connection_pool[i][j].fd;
       /* do this because keeping track is REALLY complicated */
@@ -127,9 +142,9 @@ static int reserve_session(struct in_addr *request, pid_t pid) {
     }
   return rfd;
 }
-void replace_session(struct in_addr *submission, int fd) {
+void replace_session(struct in_addr *submission, unsigned short port, int fd) {
   int i,j;
-  if((i=find_server(submission))==-1) {
+  if((i=find_server(submission, port))==-1) {
     if(loglevel & MBLL_NET3) {
       ap_log_error(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, NULL,
 		   "mod_backhand: Sent session to unknown server: close(%d)",
@@ -191,9 +206,9 @@ void html_print_serverstats_table(request_rec *r) {
 		   (int)(time(NULL) - serverstats[i].mtime), /* Age */
 		   hostaddress,                /* aaa.bbb.ccc.ddd:port */
 		   /* Total physical memory in megabytes */
-		   ((unsigned int)serverstats[i].tmem)/(1024*1024),
+		   (unsigned int)serverstats[i].tmem,
 		   /* Avilable memory in megabytes */
-		   ((unsigned int)serverstats[i].amem)/(1024*1024),
+		   (unsigned int)serverstats[i].amem,
 		   serverstats[i].aservers,    /* # ready Apache procs */
 		   serverstats[i].nservers,    /* # of Apache procs */
 		   serverstats[i].tatime,      /* Ave ms / request */
@@ -615,11 +630,14 @@ IP_MULTICAST_TTL for server statistic multicasting.");
 	    children[i].fd = children[i].pid = -1;
 	  } else {
 	    /* Child is giving us or asking for a session */
+	    unsigned short portreq;
 	    memcpy(&servreq, chreq+1, sizeof(struct in_addr));
+	    memcpy(&portreq, chreq+1+sizeof(struct in_addr),
+		   sizeof(unsigned short));
 	    if(*chreq == MBCSP_GET) {
 	      /* We need to send them a session */
 	      int reqfd;
-	      reqfd = reserve_session(&servreq, children[i].pid);
+	      reqfd = reserve_session(&servreq, portreq, children[i].pid);
 	      send_fd(children[i].fd, reqfd);
               close(reqfd);
 	      if(loglevel & MBLL_NET1) {
@@ -635,7 +653,7 @@ IP_MULTICAST_TTL for server statistic multicasting.");
 			     "mod_backhand: Receiving back fd(??).");
 	      }
 	      if((recvdfd = recv_fd(children[i].fd))>0)
-		replace_session(&servreq, recvdfd);
+		replace_session(&servreq, portreq, recvdfd);
 	      if(loglevel & MBLL_NET1) {
 		ap_log_error(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, NULL,
 			     "mod_backhand: Received back fd(%d).",
