@@ -26,7 +26,7 @@ typedef struct {
 
 void html_print_serverstats_table(request_rec *);
 int broadcast_my_stats(void *, child_info *);
-int find_server(struct in_addr *sin);
+int find_server(struct in_addr *sin, unsigned short);
 ACL *matchACL(ACL *acl, struct in_addr *in);
 void backhand_initstat(void);
 
@@ -39,11 +39,12 @@ void backhand_initstat(void);
 /* client/server protocol:
    1) client->server GET/PUT/CLOSE (1-byte)
    2) client->server WHAT (sizeof(struct in_addr))
+   2) client->server WHAT (sizeof(unsigned short))
    3(?GET) server->client (transfer fd)
    3(?PUT) client->server (transfer fd)
 */
 #define MBCSP_GET 0
 #define MBCSP_PUT 1
 #define MBCSP_CLOSE 2
-#define MBCSP_REQ_SIZE (1+sizeof(struct in_addr))
+#define MBCSP_REQ_SIZE (1+sizeof(struct in_addr)+sizeof(unsigned short))
 #endif
