@@ -186,15 +186,27 @@ static const char *setupbroadcasting(cmd_parms *cmd, void *mconfig,
     }
     memcpy(&from, he->h_addr_list[0], sizeof(struct in_addr));    
   } else if(word1) {
-    if(strchr(word1, ':')) {
-      sscanf(word1, "%d.%d.%d.%d:%u", &a, &b, &c, &d, &webport);
-      if(webport)
-	from.s_addr = htonl((a << 24) | (b << 16) | (c << 8) | d);
-      webport=htons(webport);
+    struct hostent *he;
+    char *cp;
+    if((cp = strchr(word1, ':')) != NULL) {
+      *cp = '\0';
+      if((he = gethostbyname(word1)) != NULL) {
+	memcpy(&from, he->h_addr_list[0], sizeof(struct in_addr));
+      } else {
+        sscanf(word1, "%d.%d.%d.%d", &a, &b, &c, &d);
+	webport = atoi(cp+1);
+        if(webport)
+	  from.s_addr = htonl((a << 24) | (b << 16) | (c << 8) | d);
+        webport=htons(webport);
+      }
     } else {
-      sscanf(word1, "%d.%d.%d.%d", &a, &b, &c, &d);
-      if(d)
-	from.s_addr = htonl((a << 24) | (b << 16) | (c << 8) | d);
+      if((he = gethostbyname(word1)) != NULL) {
+	memcpy(&from, he->h_addr_list[0], sizeof(struct in_addr));
+      } else {
+        sscanf(word1, "%d.%d.%d.%d", &a, &b, &c, &d);
+        if(d)
+	  from.s_addr = htonl((a << 24) | (b << 16) | (c << 8) | d);
+      }
     }
   }
   if(strchr(word2, ',')) {
@@ -205,14 +217,18 @@ static const char *setupbroadcasting(cmd_parms *cmd, void *mconfig,
       sin.sin_family = ttl;
       sin.sin_addr.s_addr = htonl((a << 24) | (b << 16) | (c << 8) | d);
       sin.sin_port = htons(port);
-    }
+    } else
+      return "MulticastStats ip:port,ttl syntax error";
   } else {
     if(word2) sscanf(word2, "%d.%d.%d.%d:%d", &a, &b, &c, &d, &port);
     if(port!=0) {
       sin.sin_family = AF_INET;
       sin.sin_addr.s_addr = htonl((a << 24) | (b << 16) | (c << 8) | d);
       sin.sin_port = htons(port);
-    }
+    } else
+      return "MulticastStats ip:port syntax error";
+    if(ismulticast(sin))
+      return "multicast address must include ttl (ip:port,ttl)";
   }
   if(sin.sin_port) {
     for(i=0;i<nsins;i++)
@@ -445,7 +461,7 @@ static const char *cmd_backhand_so(cmd_parms *cmd, void *mconfig, char *word1,
 static int build_request_headers(void *nuttin,
 				 const char *key, const char *val) {
   char buffer[DEFAULT_LIMIT_REQUEST_FIELDSIZE + 2];
-  ap_snprintf(buffer, DEFAULT_LIMIT_REQUEST_FIELDSIZE+2, "%s: %s\n", key, val);
+  ap_snprintf(buffer, DEFAULT_LIMIT_REQUEST_FIELDSIZE+2, "%s: %s" CRLF, key, val);
   strcat(nuttin, buffer);
   return 1;
 }
@@ -593,7 +609,7 @@ static int backhand_redirection(request_rec *r) {
 
     /* Build our request */
     tempaddr = r->connection->remote_addr.sin_addr;
-    ap_snprintf(header, sizeof header, "%s\n", r->the_request);
+    ap_snprintf(header, sizeof header, "%s" CRLF, r->the_request);
     clientka = ap_find_token(r->pool,
 			     ap_table_get(r->headers_in, "Connection"),
 			     "Keep-Alive");
@@ -603,7 +619,7 @@ static int backhand_redirection(request_rec *r) {
     ap_table_setn(r->headers_in, "BackhandProxied", inet_ntoa(tempaddr));
     ap_table_do(build_request_headers, header, r->headers_in, NULL);
     ap_table_unset(r->headers_in, "BackhandProxied");
-    strcat(header, "\n");
+    strcat(header, CRLF);
     
     /* Get our socket */
     /* We only try once if we are not using connection pooling */
@@ -622,11 +638,15 @@ static int backhand_redirection(request_rec *r) {
 	goto local_handle;
       }
       if(usepool == MBCP_OFF) {
-	mbcs = new_session(&serverstats[remote_machine.id].contact);
+	mbcs = new_session(serverstats[remote_machine.id].contact.sin_addr.s_addr,
+			   serverstats[remote_machine.id].contact.sin_port);
       } else {
 	memcpy(mbcsp_request+1,
 	       &serverstats[remote_machine.id].contact.sin_addr,
 	       sizeof(struct in_addr));
+	memcpy(mbcsp_request+1+sizeof(struct in_addr),
+	       &serverstats[remote_machine.id].contact.sin_port,
+	       sizeof(unsigned short));
 	mbcsp_request[0] = MBCSP_GET;
 	/* Apparently this next stanza can hang on BSD?! */
 	recvfd_failed = 1;
@@ -818,7 +838,7 @@ static int backhand_redirection(request_rec *r) {
     } else if(chunked) {
       remaining=0;
       tsize=0;
-      while((size = get_response_block(myconn, &remaining, buffer, RBUFLEN))>0) {
+      while((size = get_response_block(myconn, r, &remaining, buffer, RBUFLEN))>0) {
 	tsize=size;
 	ssize=0;
 	while((ssize<tsize) && 
@@ -888,7 +908,17 @@ static int backhand_redirection(request_rec *r) {
   }
  local_handle:
   ap_kill_timeout(r);
-  ap_internal_redirect(r->uri, r);
+  if(!ap_table_get((r->main)?r->main->notes:r->notes,
+     "BackhandHandlerExecuted")) {
+    ap_table_setn((r->main)?r->main->notes:r->notes,
+                  "BackhandHandlerExecuted", "Done");
+    ap_internal_redirect(r->uri, r);
+  } else {
+    ap_log_error(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, NULL,
+		 "mod_backhand: "
+		 "attempted mod_backhand redirection twice. BUG.");
+    return DECLINED;
+  }
   if(loglevel & MBLL_MBCS3) {
     ap_log_error(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, NULL,
 		 "mod_backhand: "
@@ -966,7 +996,13 @@ static void backhand_init(server_rec *s, pool *p) {
 
   setupUDPacl(NULL,NULL,NULL);
 
-  if(getppid() == 1) {
+  if(!nsins) {
+    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_EMERG, s,
+	"No MulticastStats specified");
+    exit(1);
+  }
+
+  if(getppid()) {
     int i, afd, staterror;
     struct stat unused;
     daemon_params data;
@@ -1303,8 +1339,10 @@ static int backhand_post_read_request(request_rec *r) {
 	 tmpcfg->local && tmpcfg->congenital) {
 	const char *rhost = ap_table_get(subr->notes, "Backhand-Redirect-Host");
 	if(rhost) ap_table_set(r->notes, "Backhand-Redirect-Host", rhost);
-	r->uri = ap_pstrcat(r->pool, "backhand:", r->uri, NULL);
-	r->filename = ap_pstrdup(r->pool, r->uri);
+        if(r->uri && strncmp(r->uri, "backhand:", 9))
+	  r->uri = ap_pstrcat(r->pool, "backhand:", r->uri, NULL);
+        if(r->filename && strncmp(r->filename, "backhand:", 9))
+	  r->filename = ap_pstrcat(r->pool, "backhand:", r->filename, NULL);
 	r->handler = "backhand-redirection";
 	if(loglevel & MBLL_DCSN1) {
 	  ap_log_error(APLOG_MARK, APLOG_NOTICE|APLOG_NOERRNO, NULL,
@@ -1348,6 +1386,8 @@ static int backhand_translate_handler(request_rec *r) {
     return DECLINED;
   }
   memmove(r->uri, &r->uri[9], strlen(&r->uri[9])+1);
+  if(r->filename && strncmp(r->filename, "backhand:", 9)==0)
+    memmove(r->filename, &r->filename[9], strlen(&r->filename[9])+1);
   return OK;
 }
 
@@ -1380,8 +1420,10 @@ static int backhand_type_checker(request_rec *r) {
     if(remote_machine.id>=0 &&
        ((sredir==MBSR_ON) || (remote_machine.id>0))) {
       r->content_type = "backhand/redirect";
-      r->uri = ap_pstrcat(r->pool, "backhand:", r->uri, NULL);
-      r->filename = ap_pstrcat(r->pool, "backhand:", r->filename, NULL);
+      if(r->uri && strncmp(r->uri, "backhand:", 9))
+        r->uri = ap_pstrcat(r->pool, "backhand:", r->uri, NULL);
+      if(r->filename && strncmp(r->filename, "backhand:", 9))
+        r->filename = ap_pstrcat(r->pool, "backhand:", r->filename, NULL);
       r->handler = "backhand-redirection";
     }
   }
