@@ -1,46 +1,46 @@
-/* ======================================================================
- * Copyright (c) 1998-1999 The Johns Hopkins University.
- * All rights reserved.
- * The following code was written by Theo Schlossnagle for use in the
- * Backhand project at The Center for Networking and Distributed Systems
- * at The Johns Hopkins University.
- * Please refer to the LICENSE file before using this software.
- * ======================================================================
-*/
-/* This is exported for use in builtins.c */
-int find_highest_arriba(serverstat *ss) {
-  int i, highest=0;
-  for(i=0;i<HARD_SERVER_LIMIT;i++)
-    highest = MAX(highest,
-		  ss[i].arriba);
-  return highest;
-}
-static int find_max_load(serverstat *ss, int ha) {
-  int i, highest=0;
-  float myaa;
-  for(i=0;i<HARD_SERVER_LIMIT;i++) {
-    if(ss[i].arriba) myaa = (float)ha/(float)ss[i].arriba;
-    else myaa = 0;
-    highest = MAX(highest,
-		  (float)ss[i].load*myaa);
-  }
-  return highest;
-}
-static int set_high_watermark(void) {
-  int i=0, aload, ha;
-  ha = find_highest_arriba(serverstats);
-  aload = find_max_load(serverstats, ha);
-  if(aload>0) aload-=1; /* n - delta */
-  aload /= 1000; /* places hwm @ mag(load)*/
-  aload <<= 1;
-  while(aload >>=1) i++; /* log2(1+n-delta) */
-  aload = 1 << i; /* 2^(n+1) */
-  for(i=0;i<HARD_SERVER_LIMIT;i++)
-    aload = MAX(aload, serverstats[i].load_hwm/1000);
-  return aload;
-}
+	/* ======================================================================
+	 * Copyright (c) 1998-1999 The Johns Hopkins University.
+	 * All rights reserved.
+	 * The following code was written by Theo Schlossnagle for use in the
+	 * Backhand project at The Center for Networking and Distributed Systems
+	 * at The Johns Hopkins University.
+	 * Please refer to the LICENSE file before using this software.
+	 * ======================================================================
+	*/
+	/* This is exported for use in builtins.c */
+	int find_highest_arriba(serverstat *ss) {
+	  int i, highest=0;
+	  for(i=0;i<HARD_SERVER_LIMIT;i++)
+	    highest = MAX(highest,
+			  ss[i].arriba);
+	  return highest;
+	}
+	static int find_max_load(serverstat *ss, int ha) {
+	  int i, highest=0;
+	  float myaa;
+	  for(i=0;i<HARD_SERVER_LIMIT;i++) {
+	    if(ss[i].arriba) myaa = (float)ha/(float)ss[i].arriba;
+	    else myaa = 0;
+	    highest = MAX(highest,
+			  (float)ss[i].load*myaa);
+	  }
+	  return highest;
+	}
+	static int set_high_watermark(void) {
+	  int i=0, aload, ha;
+	  ha = find_highest_arriba(serverstats);
+	  aload = find_max_load(serverstats, ha);
+	  if(aload>0) aload-=1; /* n - delta */
+	  aload /= 1000; /* places hwm @ mag(load)*/
+	  aload <<= 1;
+	  while(aload >>=1) i++; /* log2(1+n-delta) */
+	  aload = 1 << i; /* 2^(n+1) */
+	  for(i=0;i<HARD_SERVER_LIMIT;i++)
+	    aload = MAX(aload, serverstats[i].load_hwm/1000);
+	  return aload;
+	}
 
-#ifdef LINUX
+	#ifdef LINUX
 #ifndef CPUSTATES
 #define CPUSTATES 4
 #endif
@@ -152,8 +152,11 @@ static void fillstat(serverstat *ss, server_rec *s,
       if(strncmp(dummy, "Mem:", 4)==0) {
 	int tm, um, fm, sm, bm, cm;
 	sscanf(dummy+5, "%d %d %d %d %d %d", &tm, &um, &fm, &sm, &bm, &cm);
-	tmem=tm;
-	amem=MAX(fm+bm+cm-5000000, 0); // Free + Cached + Buffered ???
+	tmem=tm/(1024*1024);
+	fm /= (1024*1024);
+	bm /= (1024*1024);
+	cm /= (1024*1024);
+	amem=MAX(fm+bm+cm-5, 0); // Free + Cached + Buffered ???
 	break;
       }
     }
@@ -235,6 +238,7 @@ void initstat(void) {
   struct kstat_list *diskcur,*cpucur, *kl;
   /* Open the kstat interface */
   pgsize = sysconf(_SC_PAGESIZE);
+  pgsize /= 1024; /* Page size in kb */
   if(NULL==(kctl=kstat_open())){
     perror("kstat");
     exit(1);
@@ -264,6 +268,7 @@ void initstat(void) {
       syspages = k;
       if(kstat_read(kctl, syspages, 0)!=-1 &&
 	 (kn = (kstat_named_t *)kstat_data_lookup(syspages, "physmem"))!=NULL)
+	kn->value.ul /= 1024; /* pgsize in kb, we want mb */
 	physmem = pgsize*kn->value.ul;
       else
         physmem = 0;
@@ -379,6 +384,7 @@ static void fillstat(serverstat *ss, server_rec *s,
   if(syspages &&
      kstat_read(kctl, syspages, 0)!=-1 &&
      ((kn = (kstat_named_t *)kstat_data_lookup(syspages, "freemem"))!=NULL))
+    kn->value.ul /= 1024; /* pgsize in kb, we want mb */
     ss->amem = pgsize*kn->value.ul;
   else
     ss->amem = 0;
@@ -438,7 +444,7 @@ void initstat(void)
       mach_error("can't get host_basic_info???\n", kr);
    
    ncpu    = (int)hostinfo.avail_cpus;   
-   usermem = (int)hostinfo.memory_size;
+   usermem = (int)hostinfo.memory_size /= 1024*1024;
       
 }
 
@@ -451,7 +457,6 @@ static void fillstat(serverstat *ss, server_rec *s,
    register float 			ourcpu = 0.0;
    vm_statistics_data_t			vminfo;
    host_load_info_data_t		loadinfo;
-   register host_cpu_load_info_t	temp;    
    count = HOST_VM_INFO_COUNT;
    kr = host_statistics(mach_host_self(), HOST_VM_INFO,
                         (host_info_t)&vminfo, &count);
@@ -475,12 +480,15 @@ static void fillstat(serverstat *ss, server_rec *s,
   ourcpu = 1.0 - (((float)(newcpuinfo.cpu_ticks[CPU_STATE_IDLE] - 
                         oldcpuinfo.cpu_ticks[CPU_STATE_IDLE])) / ourcpu);
   ourcpu *= 1000.0;
-  
+  for(i = 0; i < CPU_STATE_MAX; i++)
+    oldcpuinfo.cpu_ticks[i] = newcpuinfo.cpu_ticks[i];
+
   ss->load = (double)loadinfo.avenrun[0];
   ss->load_hwm = set_high_watermark() * 1000;
 
   ss->tmem = usermem;
 
+  vminfo.free_count /= 1024*1024; /* calculate in mb */
   ss->amem = (int)(vm_page_size * vminfo.free_count);
   
   ss->cpu = (int)ourcpu;
@@ -569,7 +577,7 @@ static void fillstat(serverstat *ss, server_rec *s,
   sysctl(mib, 2, &ourload, &len, NULL, 0);
   ss->load = (int)1000.0*((float)ourload.ldavg[0]/(float)ourload.fscale);
   ss->load_hwm=set_high_watermark()*1000;
-  ss->tmem=(int)usermem;
+  ss->tmem=(int)usermem/(1024*1024); /* in mb */
 #ifdef VM_TOTAL
   mib[1] = VM_TOTAL;
 #else
@@ -581,6 +589,7 @@ static void fillstat(serverstat *ss, server_rec *s,
 #endif
   len = sizeof(struct vmtotal);
   sysctl(mib, 2, &ourmem, &len, NULL, 0);
+  ourmem.t_free /= 1024*1024; /* calculate in mb */
   ss->amem=(int)usermem-(ourpagesize*ourmem.t_free);
   ss->cpu =(int)ourcpu;
   ss->ncpu=(int)ncpu;
